@@ -2,14 +2,13 @@ use v5.10;
 use strict;
 use warnings;
 
-use Test::More;
-use Test::Identity;
+use Test2::V0;
 
 use Future;
 
 my $f = Future->new;
 
-identical( $f->set_label( "the label" ), $f, '->set_label returns $f' );
+ref_is( $f->set_label( "the label" ), $f, '->set_label returns $f' );
 
 is( $f->label, "the label", '->label returns the label' );
 
